import os
from cryptography.fernet import Fernet

class PasswordHandler():
    key_file = os.path.join(os.getcwd(),"bigtest_automator","key.key")
    # key_file = os.path.join(key_path,"key.key")

    def key_loader():
        # Generate or load the key
        if os.path.exists(PasswordHandler.key_file):
            with open(PasswordHandler.key_file, "rb") as f:
                key = f.read()
        else:
            key = Fernet.generate_key()
            with open(PasswordHandler.key_file, "wb") as f:
                f.write(key)

        # Create a Fernet object with the key
        return Fernet(key)


    def encrypt_password(plaintext_password):
        fernet = PasswordHandler.key_loader()
        return fernet.encrypt(plaintext_password.encode()).decode("utf-8")
        

    def decrypt_password(encrypted_password):
        try:
            fernet = PasswordHandler.key_loader()
            return fernet.decrypt(encrypted_password).decode("utf-8")
        except Exception as e:
            print(e)
            raise ValueError("Keys did not match! Did you modify key.key file?")
            #return None




import db_ops as db

insert = "INSERT INTO SERVERS (NAME, SERVER_IP, USER_NAME, AUTH_METHOD, PASSWORD) VALUES ('Shantharam', '10.0.0.1','shantharamp','password','dummy');"
selectq = "SELECT * FROM SERVERS;"
delete_all = "DELETE FROM SERVERS;"

select = db.query_executor(selectq)

for item in select:
    print(item)

db.query_executor(delete_all)
#db.query_executor(insert)

sec_select = db.query_executor(selectq)

for item in sec_select:
    print(item)


#takes in IDs at once, returns a a list of list with details.
#query
#SELECT ID, NAME, SERVER_IP, USER_NAME, AUTH_METHOD, KEY_FILE_LOCATION, PASSWORD FROM SERVERS WHERE ID IN ();
from internal_db_management import db_ops as db
from internal_db_management.pwd_handler import PasswordHandler

class GetServerDetails():
    def get_server_details(server_list,table_name):
        if table_name == "SERVERS":
            query = ""
            if(len(server_list)==1):
                query = "SELECT ID, NAME, SERVER_IP, USER_NAME, AUTH_METHOD, KEY_FILE_LOCATION, PASSWORD FROM SERVERS WHERE ID IN ("+server_list[0]+");"
            else:
                id_tuple = tuple(server_list)
                query = "SELECT ID, NAME, SERVER_IP, USER_NAME, AUTH_METHOD, KEY_FILE_LOCATION, PASSWORD FROM SERVERS WHERE ID IN "+str(id_tuple)+";"

        
        elif table_name == "VMSTAT":
            query = ""
            if(len(server_list)==1):
                query = """SELECT
                            S.ID, S.NAME, S.SERVER_IP, S.USER_NAME, S.AUTH_METHOD, S.KEY_FILE_LOCATION, S.PASSWORD, V.COMMON_NAME
                            FROM
                            SERVERS S, VMSTAT V WHERE
                            S.ID = V.SERVER AND
                            V.ID IN ({});
                        """.format(server_list[0])
            else:
                id_tuple = tuple(server_list)
                query = """SELECT
                            S.ID, S.NAME, S.SERVER_IP, S.USER_NAME, S.AUTH_METHOD, S.KEY_FILE_LOCATION, S.PASSWORD, V.COMMON_NAME
                            FROM
                            SERVERS S, VMSTAT V WHERE
                            S.ID = V.SERVER AND
                            V.ID IN {};
                        """.format(str(id_tuple))
                
        elif table_name == "GEN_AWR":
            id_tuple = tuple(server_list)
            query = """
                SELECT S.ID, S.NAME, S.SERVER_IP, S.USER_NAME, S.AUTH_METHOD, S.KEY_FILE_LOCATION, S.PASSWORD, A.SNAP_ID
                FROM 
                SERVERS S, AWR A WHERE
                A.SERVER_ID = S.ID AND
                A.ID IN {}
            """.format(str(id_tuple))

        rs = db.query_executor(query)
        rs_list = list(rs.fetchall())
        for each_server_list in range(len(rs_list)):
            rs_list[each_server_list] = list(rs_list[each_server_list])
            if rs_list[each_server_list][4] == 'Password':
                rs_list[each_server_list][6] = PasswordHandler.decrypt_password(rs_list[each_server_list][6])

        return rs_list





import os,sqlite3

def create_tables(db_file):

    create_servers_table = """
        CREATE TABLE IF NOT EXISTS SERVERS (
        ID integer PRIMARY KEY AUTOINCREMENT,
        NAME text NOT NULL,
        SERVER_IP text NOT NULL,
        USER_NAME text NOT NULL,
        AUTH_METHOD text NOT NULL,
        KEY_FILE_LOCATION text,
        PASSWORD text
    )
    """
    create_vmstat_table = """
        CREATE TABLE IF NOT EXISTS VMSTAT ( 
    	ID integer PRIMARY KEY AUTOINCREMENT,
    	EXECUTION_TIME text NOT NULL,
    	COMMON_NAME text NOT NULL,
    	COMMAND text NOT NULL,
    	SERVER text NOT NULL)
    """
    create_awr_table = """
        CREATE TABLE IF NOT EXISTS AWR ( 
        ID integer PRIMARY KEY AUTOINCREMENT, 
        SERVER_ID text NOT NULL, 
        SNAP_ID text NOT NULL, 
        SNAP_TIME text NOT NULL 
    )
    """
    try:
        con = sqlite3.connect(db_file,isolation_level=None)
        cursor = con.cursor()
        print("Creating table SERVERS")
        cursor.execute(create_servers_table)
        print("Creating table VMSTAT")
        cursor.execute(create_vmstat_table)
        print("Creating table AWR")
        cursor.execute(create_awr_table)
        con.commit()

    except Exception as e:
        print(e)
        print("Failed to execute DDLs")

    finally:
        if cursor:
            cursor.close()
        if con:
            con.close()

def initialize_db():
    if not os.path.exists(os.path.join(os.getcwd(),"bigtest_automator","serverdb.db")):
        os.mkdir(os.path.join(os.getcwd(),"bigtest_automator"))
        db_file = os.path.join(os.getcwd(),"bigtest_automator","serverdb.db")
        print("Creating database file: "+db_file)
        with open(db_file, 'w') as fp:
            pass
        create_tables(db_file)




from tkinter import *
from tkinter import ttk, messagebox
from tkinter import filedialog as f
import os,re
from internal_db_management import db_ops as db
from core_utils import run_vmstat as rv
from core_utils import run_scp as rscp
from internal_db_management.pwd_handler import PasswordHandler
from core_utils.take_awr_snapshot import TakeSnapshot
from core_utils.generate_awr import GenerateAWR
from ui_management import grid

g = grid.Grid() 

class ServerManagement:

    selected_servers = []
    selected_vmstat_servers = []
    type_of_auth = ""
    password = ""
    keyfile_location = ""
    action_item = ""

    def go_to_next_page(self,root,action_item):

        if(len(ServerManagement.selected_servers)==0):
            messagebox.showerror("Error!","You must select at least one server from the list to continue")
        
        else:
            # RecordActions(root,ServerManagement.selected_servers)
            InvokeActions(root,action_item)

    def exit_window(self,root):
        root.quit()

    def __init__(self,root,action_item):
        ServerManagement.action_item = action_item
        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)

        self.get_available_servers(root)
        next_button = Button(root,text="Next>",command=lambda:self.go_to_next_page(root,action_item))
        next_button.grid(row=300,column=3)
        exit_window = Button(root,text="Exit",command=lambda:self.exit_window(root))
        exit_window.grid(row=300,column=4)
            
    def record_selection(self,mylist):
        ServerManagement.selected_servers.clear()
        ServerManagement.selected_servers = mylist

    def get_available_servers(self,root):
        
        for item in root.winfo_children():
            item.destroy()

        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)

        Label(root, text="Sever Management", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root, text="Select the servers that you want to run the reports on:",font=('calibre','15','normal')).grid(row=g.get_next_row(),column=0,sticky="w")
        same_row = g.get_next_row()
        add_button = Button(root,text="Add a Server", command=lambda:self.add_a_server(root)).grid(row=same_row,column=1)
        remove_button = Button(root,text="Delete a Server", command=self.delete_a_server).grid(row=same_row,column=2)
        refresh_button = Button(root,text="Refresh", command=lambda:self.refresh(root)).grid(row=same_row,column=3)

        def getServers(event):
            self.record_selection(list(tv.selection()))

        rs = db.query_executor("SELECT ID, NAME, SERVER_IP, USER_NAME, AUTH_METHOD FROM SERVERS")
        same_row = g.get_next_row()

        tv = ttk.Treeview(root, columns=(1,2,3,4,5),show='headings')
        tv.grid(row=g.get_next_row(),columnspan=10,sticky='nsew',rowspan=100)

        tv.heading(1, text='ID')
        tv.heading(2, text='Name')
        tv.heading(3, text='Server IP')
        tv.heading(4, text='Username')
        tv.heading(5, text='Type of Authentication')


        for rows in rs:
            tv.insert('','end',iid=rows[0],values=rows)

        tv.bind("<<TreeviewSelect>>",getServers)

    def add_a_server(self,root):

        def decide_auth_method(event):

            if(keyfileMenu.get() == "Password"):

                def get_password(event):
                    ServerManagement.password = password_entry.get()
                   
                ServerManagement.type_of_auth = "Password"
                authtype_menu.destroy()
                Label(addWindow,text="Enter the password: ",pady=10).grid(row=4,column=0)
                password_entry = Entry(addWindow,width=40, show="*", borderwidth=2)
                password_entry.grid(row=4,column=1,padx=10,pady=10)
                addWindow.bind('<Leave>', get_password)

            elif(keyfileMenu.get()=="Private Key File"):

                ServerManagement.type_of_auth = "Private Key File"
                keyfile_label = Label(addWindow,text="Select the keyfile: ",pady=10)
                keyfile_label.grid(row=4,column=0)

                def get_keyfile():
                    private_key_path = f.askopenfilename(title = "Select a Key File")

                    if private_key_path.endswith('.pem') or private_key_path.endswith('.ppk'):
                        ServerManagement.keyfile_location = os.path.abspath(private_key_path)
                        file_dialog_button.destroy()
                        keyfile_label.destroy()
                        Label(addWindow,text="File chosen:"+ServerManagement.keyfile_location).grid(row=3,column=1)

                    else:
                        messagebox.showerror("Error!",message="Only .pem or .ppk files are allowed!")
                        private_key_path = ""
                        # get_keyfile()

                file_dialog_button = Button(addWindow,text="Browse", command=get_keyfile)
                file_dialog_button.grid(row=4,column=1)

                # file_dialog_button.destroy()
                # keyfile_label.destroy()
                # Label(addWindow,text="File chosen:"+ServerManagement.keyfile_location).grid(row=3,column=1)

        def submit_info():
            ip_regex = "^((25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.){3}(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])$"

            if len(name_entry.get())<5 or len(ip_entry.get())<5 or len(username_entry.get())<5:
                messagebox.showerror("Error!", message="Please check your inputs. Minimum character length is 5")
                     
            elif not re.search(ip_regex,ip_entry.get()):
                messagebox.showerror("Error!",message="Enter a valid IP!")
            
            elif(' ' in name_entry.get() or ' ' in username_entry.get()):
                messagebox.showerror("Error!",message="You cannot have a space in username or server name")
                
            else:
                query_string=""
                if(ServerManagement.type_of_auth == "Password"):
                    if(len(ServerManagement.password)<4):
                        messagebox.showerror("Error!",message="Less than 4 characters for password? Enter a valid password!")
                    else:
                        query_string = """
                        INSERT INTO SERVERS (NAME, SERVER_IP, USER_NAME, AUTH_METHOD, PASSWORD) 
                        VALUES ('{name}','{ip}','{username}','{authmethod}','{password}');
                        """.format(name=name_entry.get(),ip=ip_entry.get(),username=username_entry.get(),authmethod=ServerManagement.type_of_auth, password=PasswordHandler.encrypt_password(ServerManagement.password))

                    
                elif(ServerManagement.type_of_auth == "Private Key File"):
                    query_string = """
                    INSERT INTO SERVERS (NAME, SERVER_IP, USER_NAME, AUTH_METHOD, KEY_FILE_LOCATION) 
                    VALUES ('{name}','{ip}','{username}','{authmethod}','{keyfilelocation}');
                    """.format(name=name_entry.get(),ip=ip_entry.get(),username=username_entry.get(),authmethod=ServerManagement.type_of_auth, keyfilelocation=ServerManagement.keyfile_location)
                
                if(len(query_string)>1):
                    db.query_executor(query_string)
                    addWindow.grab_release()
                    addWindow.destroy()
                

        addWindow = Toplevel(root)
        addWindow.title("Add a New Server")
        addWindow.grab_set()

        Label(addWindow,text="Name: ",pady=10).grid(row=0,column=0)
        Label(addWindow,text="IP Address: ",pady=10).grid(row=1,column=0)
        Label(addWindow,text="Username: ",pady=10).grid(row=2,column=0)
        Label(addWindow,text="Authentication Type: ",pady=10).grid(row=3,column=0)

        #Create a dropdown Menu
        menu_list = ["Private Key File", "Password"]
        keyfileMenu= StringVar(addWindow)
        keyfileMenu.set("Authentication Method")

        authtype_menu = OptionMenu(addWindow, keyfileMenu, *menu_list, command=decide_auth_method)

        name_entry = Entry(addWindow,width=40,borderwidth=2)
        name_entry.grid(row=0,column=1,padx=10,pady=10)
        ip_entry = Entry(addWindow,width=40,borderwidth=2)
        ip_entry.grid(row=1,column=1,padx=10,pady=10)
        username_entry = Entry(addWindow,width=40,borderwidth=2)
        username_entry.grid(row=2,column=1,padx=10,pady=10)

        authtype_menu.grid(row=3,column=1,padx=10,pady=10)

        Button(addWindow,text="Submit",command=submit_info).grid(row=6,column=0)
        Button(addWindow,text="Cancel",command=addWindow.destroy).grid(row=6,column=1)

    def refresh(self,root):
        ServerManagement.selected_servers.clear()
        for item in root.winfo_children():
            item.destroy()
        Label(root, text="Welcome to VMSTAT automator!", font=('calibre','20','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        self.__init__(root,ServerManagement.action_item)

    def delete_a_server(self):

        if(len(self.selected_servers)==0):
            messagebox.showerror("Error","Please select at least one server to delete", icon="error")
        else:
            mb = messagebox.askquestion("Delete","Are you sure about deleting {} selected server(s) from the database?".format(str(len(self.selected_servers))),icon="warning")
            if mb == 'yes':
                inclause = ""
                for iter in range(len(self.selected_servers)):
                    inclause = inclause + "," + str(self.selected_servers[iter]) 
                inclause = inclause[1:]
                query = "DELETE FROM SERVERS WHERE ID IN ({});".format(inclause)
                db.query_executor(query)


class RecordActions:

    def __init__(self,root) -> None:


        for item in root.winfo_children():
            item.destroy()

        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)

        def go_to_next_page(root):
            if(radioVar.get()==""):
                messagebox.showerror("Error!","Choose an action to proceed further")
            else:
                # InvokeActions(root,radioVar.get())
                if radioVar.get() == "runvmstat" or radioVar.get() == "runawr":
                    ServerManagement(root,radioVar.get())
                else:
                    InvokeActions(root,radioVar.get())

        def exit_window(root):
            root.quit()

        Label(root, text="Select Action", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root, text="Select a common action that you want to perform ", font=('calibre','12','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        next_button = Button(root,text="Next>",command=lambda:go_to_next_page(root))
        next_button.grid(row=300,column=3)
        exit = Button(root,text="Exit",command=lambda:exit_window(root))
        exit.grid(row=300,column=4)

        # i=1
        # for server in range(len(server_list)):
        #     rs = db.query_executor("SELECT NAME, SERVER_IP FROM SERVERS WHERE ID = {}".format(server_list[server]))
        #     for item in rs:
        #         #listbox.insert(i,str(i)+". "+item[0]+" ("+item[1]+")")
        #         Label(root,text=str(i)+". "+item[0]+" ("+item[1]+")").grid(row=g.get_next_row(),column=0,sticky="w")
        #         i += 1
        # #some additional space
        # g.get_blank_lines(root,2)

        def record_user_action():
            pass
            
        radioVar = StringVar(value='runvmstat')
        r1 = Radiobutton(root,text="Run VMSTAT",variable=radioVar,value="runvmstat",command=record_user_action)
        r1.grid(row=g.get_next_row(),column=0,sticky="news")
        r2 = Radiobutton(root,text="Collect VMSTAT Report",variable=radioVar,value="collectvmstat",command=record_user_action)
        r2.grid(row=g.get_next_row(),column=0,sticky="news")
        r3 = Radiobutton(root,text="Take a DB snapshot",variable=radioVar,value="runawr",command=record_user_action)
        r3.grid(row=g.get_next_row(),column=0,sticky="news")
        r4 = Radiobutton(root,text="Generate an AWR Report",variable=radioVar,value="collectawr",command=record_user_action)
        r4.grid(row=g.get_next_row(),column=0,sticky="news")


class InvokeActions:
    when_to_run=""
    how_much_to_wait = ""
    begin_snap_id = ""
    end_snap_id = ""
    selected_snap_list = []

    def __init__(self,root,action_item) -> None:

        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)

        if(action_item == "runvmstat"):

            # for item in root.winfo_children():
            #     item.destroy()
            # root.rowconfigure(0, weight=1)
            # root.columnconfigure(0, weight=1)

            def submit_info():
                if not duration_entry.get().isdigit() or not frequency_entry.get().isdigit():
                    messagebox.showerror("Error","Please check your input. Duration and Frequency must be a number")
                elif(len(name_entry.get())<5 or name_entry.get().isdigit() or ' ' in name_entry.get()):
                    messagebox.showerror("Error","Please enter a valid name for this report, with length greater than 5 and has at least one alphabet and must not have any space between them")
                else:
                    rv.RunVmstat(root,ServerManagement.selected_servers,duration_entry.get(),frequency_entry.get(),name_entry.get())
                

            Label(root, text="Run VMSTAT", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
            Label(root, text="Enter a few more details to proceed:", font=('calibre','13','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
            dur_grid_row = g.get_next_row()
            freq_grid_row = g.get_next_row()
            name_grid_row = g.get_next_row()
            Label(root,text="Total Test Duration in Minutes: ",pady=10).grid(row=dur_grid_row,column=0,sticky="w")
            Label(root,text="Frequency: ",pady=10).grid(row=freq_grid_row,column=0,sticky="w")
            Label(root,text="A unique name for this test: ",pady=10).grid(row=name_grid_row,column=0,sticky="w")

            duration_entry = Entry(root,width=40,borderwidth=2)
            duration_entry.grid(row=dur_grid_row,column=1,padx=10,pady=10)
            frequency_entry = Entry(root,width=40,borderwidth=2)
            frequency_entry.grid(row=freq_grid_row,column=1,padx=10,pady=10)
            name_entry = Entry(root,width=40,borderwidth=2)
            name_entry.grid(row=name_grid_row,column=1,padx=10,pady=10)

            button_grid_row = g.get_next_row()
            Button(root,text="Submit",command=submit_info).grid(row=button_grid_row,column=0)
            Button(root,text="Exit",command=root.quit).grid(row=button_grid_row,column=1)


        elif(action_item == "collectvmstat"):
            
            Label(root, text="Collect VMSTAT Reports", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
            Label(root, text="Here are the uncollected reports, select the servers from which you want to copy the reports and click proceed", font=('calibre','13','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
            g.get_blank_lines(root,1)

            def delete_from_vms():
                query = "DELETE FROM VMSTAT"
                delete_sure = messagebox.askyesno(title="Are you sure?",message="Are you sure that you want to delete all rows from the table?")
                if delete_sure:
                    db.query_executor(query)
                    messagebox.showinfo(message="All rows from the table VMSTAT have been deleted.")
                    root.quit()

            def collect_vmstat(vms_server_list):
                if len(ServerManagement.selected_vmstat_servers) == 0:
                    messagebox.showerror("Error","Please select at least one server to proceed", icon="error")
                else:
                    rscp.RunSCP(root,vms_server_list)

            same_row = g.get_next_row()
            Button(root,text="Proceed>", command=lambda:collect_vmstat(ServerManagement.selected_vmstat_servers)).grid(row=same_row,column=1)
            Button(root,text="DELETE ALL ROWS", command=lambda:delete_from_vms()).grid(row=same_row,column=2)

            g.get_blank_lines(root,2)
            
            def record_vms_selection(current_list):
                ServerManagement.selected_vmstat_servers.clear()
                ServerManagement.selected_vmstat_servers = current_list
                    
            def get_vms_servers(event):
                record_vms_selection(list(tv.selection()))

            rs = db.query_executor("SELECT V.ID,V.EXECUTION_TIME,V.COMMON_NAME,V.COMMAND, S.NAME, S.SERVER_IP FROM VMSTAT V, SERVERS S WHERE V.SERVER = S.ID")

            tv = ttk.Treeview(root, columns=(1,2,3,4,5,6),show='headings')
            tv.grid(row=g.get_next_row(),columnspan=10,sticky='nsew',rowspan=100)

            tv.heading(1, text='ID')
            tv.heading(2, text='Executed On')
            tv.heading(3, text='Name Provided')
            tv.heading(4, text='Command Used')
            tv.heading(5, text='Server Name')
            tv.heading(6, text='Server IP')


            for rows in rs:
                tv.insert('','end',iid=rows[0],values=rows)

            tv.bind("<<TreeviewSelect>>",get_vms_servers)
          


        elif(action_item == "runawr"):

            # ServerManagement(root)

            if len(ServerManagement.selected_servers)!=1:
                messagebox.showerror("ERROR!",message="For this option, you will have to select just one server at a time.\n Please close the window and select only one DB at a time.")
                root.quit()

            else:
                Label(root, text="Take an AWR snapshot", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
                Label(root, text="Choose when to take the snapshot", font=('calibre','13','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
                g.get_blank_lines(root,1)

                def take_awr_snapshot():
                    if awr_radio_var.get() == "now":
                        InvokeActions.when_to_run = "now"
                        TakeSnapshot(root,ServerManagement.selected_servers,InvokeActions.when_to_run, InvokeActions.how_much_to_wait)


                    elif awr_radio_var.get() == "later":
                        InvokeActions.when_to_run = "later"
                        message = """
                            You have chosen to take the snapshot later at a time. \n
                            Please note that for this to work, you must keep this program running until the time window. \n
                            Do you still want to proceed?
                        """
                        up_consent = messagebox.askyesno(title="Can you leave this open?",message=message)

                        if up_consent:

                            for item in root.winfo_children():
                                item.destroy()
                            root.rowconfigure(0, weight=1)
                            root.columnconfigure(0, weight=1)

                            def validate_take_after_minutes(minutes):
                                if not str(minutes).isdigit():
                                    messagebox.showerror("Error!",message="Enter a valid number!")
                                else:
                                    InvokeActions.how_much_to_wait = take_after_minute_entry.get()
                                    TakeSnapshot(root,ServerManagement.selected_servers,InvokeActions.when_to_run,InvokeActions.how_much_to_wait)

                            same_row = g.get_next_row()
                            Label(root,text="Take after: ",pady=10).grid(row=same_row,column=0,sticky="w")
                            take_after_minute_entry = Entry(root,width=40,borderwidth=2)
                            take_after_minute_entry.grid(row=same_row,column=1,padx=10,pady=10)
                            Label(root,text="minute(s) ",pady=10).grid(row=same_row,column=2,sticky="w")
                            Button(root,text="Proceed>", command=lambda:validate_take_after_minutes(take_after_minute_entry.get())).grid(row=g.get_next_row(),column=1)

                    else:
                        messagebox.showerror("Error","Please select at least one option to proceed", icon="error")


                awr_radio_var = StringVar()
                r1 = Radiobutton(root,text="Take immediately",variable=awr_radio_var,value="now")
                r1.grid(row=g.get_next_row(),column=0,sticky="news")
                r2 = Radiobutton(root,text="Take in sometime",variable=awr_radio_var,value="later")
                r2.grid(row=g.get_next_row(),column=0,sticky="news")
        

                Button(root,text="Next>", command=take_awr_snapshot).grid(row=g.get_next_row(),column=1)

            
        else:
            Label(root, text="Generate an AWR report", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
            Label(root, text="Here are the uncollected snapshots, select begin and end snap IDs to proceed", font=('calibre','13','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
            g.get_blank_lines(root,1)

            def delete_from_awr():
                query = "DELETE FROM AWR"
                delete_sure = messagebox.askyesno(title="Are you sure?",message="Are you sure that you want to delete all rows from the table?")
                if delete_sure:
                    db.query_executor(query)
                    messagebox.showinfo(message="All rows from the table AWR have been deleted.")
                    root.quit()

            def generate_awr():
                if len(InvokeActions.selected_snap_list) == 0:
                    messagebox.showerror("Error","Please select at least one server to proceed", icon="error")
                elif len(InvokeActions.selected_snap_list)!=2:
                    messagebox.showerror("Error",message="Please select only 2 rows:\n First row as begin snap, second as the end")
                else:
                    GenerateAWR(root,InvokeActions.selected_snap_list)

            same_row = g.get_next_row()
            Button(root,text="Proceed>", command=lambda:generate_awr()).grid(row=same_row,column=1)
            Button(root,text="DELETE ALL ROWS", command=lambda:delete_from_awr()).grid(row=same_row,column=2)

            g.get_blank_lines(root,2)
            
            def record_snap_selection(current_list):
                InvokeActions.selected_snap_list.clear()
                InvokeActions.selected_snap_list = current_list
                    
            def get_vms_servers(event):
                record_snap_selection(list(tv.selection()))

            rs = db.query_executor("SELECT A.ID, S.ID, S.NAME, A.SNAP_ID, A.SNAP_TIME FROM AWR A, SERVERS S WHERE A.SERVER_ID = S.ID")

            tv = ttk.Treeview(root, columns=(1,2,3,4,5),show='headings')
            tv.grid(row=g.get_next_row(),columnspan=10,sticky='nsew',rowspan=100)

            tv.heading(1, text='ID')
            tv.heading(2, text='Server ID')
            tv.heading(3, text='Server Name')
            tv.heading(4, text='Snap ID')
            tv.heading(5, text='Snap Time')

            for rows in rs:
                tv.insert('','end',iid=rows[0],values=rows)

            tv.bind("<<TreeviewSelect>>",get_vms_servers)



from tkinter import *
from tkinter import messagebox
import tkinter.messagebox as mb
import tkinter.scrolledtext as st
from tkinter import filedialog as f
import os
from ui_management import dynamic_logger
from internal_db_management import db_ops
from internal_db_management import get_server_details
from oracle_utils import oracle_awr
from ui_management import grid

g = grid.Grid()

class GenerateAWR:
    destination = ""
    full_file_name = ""

    def __init__(self,root,snap_list) -> None:

        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)


        def get_keyfile():
            if len(filename_entry.get()) < 5 or str(filename_entry.get()).isdigit():
                messagebox.showerror("Error",message="Enter a valid filename with at least 5 alphanumerical characters")
            else:
                dest = f.askdirectory(title = "Select a Destination")
                full_file_path = os.path.abspath(dest)
                GenerateAWR.full_file_name = os.path.join(full_file_path,filename_entry.get()+".html")
                file_dialog_button.destroy()
                keyfileLabel.destroy()
                Label(root,text="File chosen:"+GenerateAWR.full_file_name).grid(row=file_chosen_row,column=1)

        Label(root, text="Generate an AWR", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root, text="DB Connection Details", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root, text="Enter the following details to connect to the DB", font=('calibre','13','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        g.get_blank_lines(root,1)


        db_user_grid_row = g.get_next_row()
        db_pass_grid_row = g.get_next_row()
        db_port_grid_row = g.get_next_row()
        db_sid_grid_row = g.get_next_row()
        file_name_row = g.get_next_row()
        directory_row = g.get_next_row()
        file_chosen_row = g.get_next_row()


        Label(root,text="Database username: ",pady=10).grid(row=db_user_grid_row,column=0,sticky="w")
        Label(root,text="Database password: ",pady=10).grid(row=db_pass_grid_row,column=0,sticky="w")
        Label(root,text="Port: ",pady=10).grid(row=db_port_grid_row,column=0,sticky="w")
        Label(root,text="SID: ",pady=10).grid(row=db_sid_grid_row,column=0,sticky="w")
        Label(root,text="Provide a valid filename for your report. DO NOT APPEND .HTML: ",pady=10).grid(row=file_name_row,column=0)
        filename_entry = Entry(root,width=40,borderwidth=2)
        filename_entry.grid(row=file_name_row,column=1)
        keyfileLabel = Label(root,text="Select the target directory to store the report: ",pady=10)
        keyfileLabel.grid(row=directory_row,column=0)
        file_dialog_button = Button(root, text="Browse", command=get_keyfile)
        file_dialog_button.grid(row=directory_row,column=1)


        username_entry = Entry(root,width=40,borderwidth=2)
        username_entry.grid(row=db_user_grid_row,column=1,padx=10,pady=10)
        password_entry = Entry(root,show='*',width=40,borderwidth=2)
        password_entry.grid(row=db_pass_grid_row,column=1,padx=10,pady=10)
        port_entry = Entry(root,width=40,borderwidth=2)
        port_entry.grid(row=db_port_grid_row,column=1,padx=10,pady=10)
        sid_entry = Entry(root,width=40,borderwidth=2)
        sid_entry.grid(row=db_sid_grid_row,column=1,padx=10,pady=10)


        def submit_info():
            username = username_entry.get()
            password = password_entry.get()
            port = port_entry.get()
            sid = sid_entry.get()

            if not port.isdigit():
                messagebox.showerror("Error", message="Please check your input. Port must be a number")
            elif(' ' in username or ' ' in password or ' ' in sid):
                messagebox.showerror("Error", message="Please enter valid values")
            elif(len(username) < 1 or len(password) < 1  or len(sid) < 1 ):
                messagebox.showerror("Error", message="Please enter valid values")
            else:
                full_server_detail = get_server_details.GetServerDetails.get_server_details(snap_list,"GEN_AWR")
                if(full_server_detail[0][0] != full_server_detail[1][0]):
                    messagebox.showerror("Error",message="The begin and end snap ID are not from the same database!")
                else:

                    def write_transaction_log():
                        query = "DELETE FROM AWR WHERE SNAP_ID IN ('{}','{}')".format(full_server_detail[0][7],full_server_detail[1][7])
                        db_ops.query_executor(query)

                    def exit_window():
                        answer = mb.askyesno(title='Are you sure?', message="Are you sure that you want to exit? Possible transaction loss if you proceed when a transaction is in progress.")
                        if answer:
                            root.quit()

                    for item in root.winfo_children():
                        item.destroy()

                    root.rowconfigure(0, weight=1)
                    root.columnconfigure(0, weight=1) 

                    Label(root, text="Generating the AWR...", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
                    logtext = st.ScrolledText(root, width = 75, height = 30, font = ("DejaVu Sans Mono",12))
                    logtext.grid(row=g.get_next_row(),column=0,columnspan=10)
                    dynamic_logger.Logger(root,logtext,"Running the command...")
                    Button(root,text="Exit",command=exit_window).grid(row=g.get_next_row(),column=1)

                    print(full_server_detail[0], username, password, port, sid,full_server_detail[0][7],full_server_detail[1][7],GenerateAWR.full_file_name)
                    outfile, logger = oracle_awr.generate_awr(root, full_server_detail[0][2], username, password, port, sid,full_server_detail[0][7],full_server_detail[1][7],GenerateAWR.full_file_name)

                    if outfile == "error":
                        dynamic_logger.Logger(root,logtext,"Could not generate the AWR report")
                        for log in logger:
                            dynamic_logger.Logger(root,logtext,log)
                    
                    else:
                        for log in logger:
                            dynamic_logger.Logger(root,logtext,log)
                        dynamic_logger.Logger(root,logtext,"Successfully generated the AWR at:"+outfile)
                        dynamic_logger.Logger(root,logtext,"These snaps have been erased from the local table")
                        write_transaction_log()

        button_grid_row = g.get_next_row()
        Button(root,text="Submit",command=submit_info).grid(row=button_grid_row,column=0)
        Button(root,text="Exit",command=root.quit).grid(row=button_grid_row,column=1)


import oracledb

def connect_to_oracle_db(server,user,password,port,sid):
    try:
        print(server, user, password, port, sid)
        params = oracledb.ConnectParams(host=server, port=port, service_name=sid)
        connection = oracledb.connect(user=user, password=password, params=params)
        return connection, connection.cursor() 

    except Exception as e:
        print(e)
        raise ValueError("Error connecting to the database")

def execute_query(type,query,server,user,password,port,sid):
    connection= ""
    cursor = ""
    try:
        result_set = []
        
        connection, cursor = connect_to_oracle_db(server,user,password,port,sid)

        if type == "plsql":
            cursor.callproc("dbms_output.enable")
            chunk_size = 1000
            lines_var = cursor.arrayvar(str, chunk_size)
            num_lines_var = cursor.var(int)
            num_lines_var.setvalue(0, chunk_size)
            cursor.execute(query)
            while True:
                cursor.callproc("dbms_output.get_lines", (lines_var, num_lines_var))
                num_lines = num_lines_var.getvalue()
                lines = lines_var.getvalue()[:num_lines]
                result_set.append(lines)
                return result_set
        
        else:
            rs = cursor.execute(query)
            connection.commit()
            for row in rs:
                result_set.append(row)
            return result_set
    
    except Exception as e:
        print(e)
        raise ValueError("Error while executing the query")

    finally:
        if cursor:
            cursor.close()
        if connection:
            connection.close()


def generate_awr(server,user,password,port,sid,begin_snap,end_snap):
    try:
        return_awr_report = ""

        connection, cursor = connect_to_oracle_db(server,user,password,port,sid)
        cursor.execute("SELECT DBID FROM V$DATABASE")
        db_id = cursor.fetchone()[0]

        query = """
                SELECT output FROM TABLE
                (dbms_workload_repository.awr_report_html
                    ({},{},{},{})
                )
                """.format(db_id,str(1),begin_snap,end_snap)
        
        cursor.execute(query)
        result = cursor.fetchall()

        return_awr_report = result
        
        return return_awr_report
    
    except Exception as e:
        print(e)
        raise ValueError("Error while generating the AWR report")
    
    finally:
        if cursor:
            cursor.close()
        if connection:
            connection.close()


from tkinter import *
class Grid:

    row_counter = 0

    def get_next_row(self):
        Grid.row_counter += 1
        return Grid.row_counter

    def get_blank_lines(self,root,num_of_blank_lines):
        for _ in range(num_of_blank_lines):
            Label(root,text="").grid(row=Grid.get_next_row(self),column=0)


from oracle_utils import remote_oracle_db_connector
# from ui_management import dynamic_logger
import re
from tkinter import *
from ui_management import grid

g = grid.Grid()


def take_db_snapshot(root,server,username,password,port,sid,when_to_take,how_much_to_wait):
    logger = []

    def run_snap_query():
        try:
            # query = "exec dbms_workload_repository.create_snapshot"
            query = """
                    begin
                        dbms_output.put_line(dbms_workload_repository.create_snapshot);
                    end;
                    """
            rs = remote_oracle_db_connector.execute_query("plsql",query,server,username,password,port,sid)
            rs = rs[0][0]

            # is_success = [each_item for each_item in rs if "successfully completed" in each_item]

            if re.search("\d{5}",rs):
                logger.append("Successfully created the snapshot, fetching the snap id")
                try:
                    query = "select snap_id, snap_level, to_char(begin_interval_time, 'dd/mm/yy hh24:mi:ss') begin from dba_hist_snapshot order by 1 desc fetch first 1 rows only"
                    rs = remote_oracle_db_connector.execute_query("sql",query,server,username,password,port,sid)
                    rs = list(rs[0])
                    logger.append("Snap ID {} was generated on {}".format(rs[0],rs[2]))
                    return rs[0], rs[2], logger
                
                except Exception as e:
                    logger.append(e)
                    logger.append("Failed to fetch the snap id")
                    return "Error","Error",logger
                
            else:
                logger.append("Failed to generate a snap:")
                logger.append(rs)
                return "Error", "Error", logger

        except Exception as e:
            logger.append(e)
            logger.append("Failed to create AWR snapshot")
            return "Error","Error",logger


    snap_id, snap_time, logger = run_snap_query()
    return snap_id, snap_time, logger
    
            

def generate_awr(root,server_detail,username,password,port,sid,begin_snap,end_snap,filepath):
    logger = []
    awr_array = []
    try:
        awr = remote_oracle_db_connector.generate_awr(server_detail,username,password,port,sid,begin_snap,end_snap)
        
        for i in range(len(awr)):
            awr_array.append(awr[i][0])

        with open(filepath,'a') as file_handler:
            for each_row in awr_array:
                file_handler.write(str(each_row))

        return filepath,logger
    
    except Exception as e:
        print(e)
        return "error",["error"]
    


from tkinter import *
from ui_management import grid, dynamic_logger
from internal_db_management import db_ops
from internal_db_management.get_server_details import GetServerDetails
import paramiko
import tkinter.messagebox as mb
import concurrent.futures as cf
import tkinter.scrolledtext as st
from tkinter import filedialog as f
from scp import SCPClient

import os
g = grid.Grid()


class RunSCP():
    destination = ""
    def __init__(self,root,scp_server_list) -> None:

        def submit_destination():
            ExecuteSCPCommand(root,full_server_detail)

        def get_keyfile():
            dest = f.askdirectory(title = "Select a Destination")
            RunSCP.destination = os.path.abspath(dest)
            file_dialog_button.destroy()
            keyfileLabel.destroy()
            Label(root,text="File chosen:"+RunSCP.destination).grid(row=g.get_next_row(),column=0)
            Button(root,text="Continue",command=submit_destination).grid(row=g.get_next_row(),column=0)


        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1) 

        # Label(root, text="Collecting VMSTAT...", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        full_server_detail = GetServerDetails.get_server_details(scp_server_list,"VMSTAT")

        Label(root, text="Select Destination", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root,text="Select a common destination to store your files: ",pady=10).grid(row=g.get_next_row(),column=0)
        keyfileLabel = Label(root,text="Select the directory: ",pady=10)
        keyfileLabel.grid(row=g.get_next_row(),column=0)

        file_dialog_button = Button(root, text="Browse", command=get_keyfile)
        file_dialog_button.grid(row=g.get_next_row(),column=0)

        
class ExecuteSCPCommand():
    success = True
    failed_server = []

    def __init__(self,root,full_server_detail) -> None:

        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1) 

        def write_transaction_log(full_server_detail):
            for each_server in full_server_detail:
                COMMON_NAME = each_server[7]
                query = "DELETE FROM VMSTAT WHERE COMMON_NAME='{}'".format(COMMON_NAME)
                print(query)
                db_ops.query_executor(query)

        
        def exit_window():
            answer = mb.askyesno(title='Are you sure?', message="Are you sure that you want to exit? Possible transaction loss if you proceed when a transaction is in progress.")
            if answer:
                root.quit()

        logtext = st.ScrolledText(root, width = 75, height = 30, font = ("DejaVu Sans Mono",12))
        logtext.grid(row=g.get_next_row(),column=0,columnspan=10)
        dynamic_logger.Logger(root,logtext,"Getting the files...")
        Button(root,text="Exit",command=exit_window).grid(row=g.get_next_row(),column=1)
    
        def ssh_connect(each_server):
            logger = []
            ID, NAME, SERVER_IP, USER_NAME, AUTH_METHOD, KEY_FILE_LOCATION, PASSWORD, COMMON_NAME = each_server
            remote_source_file = "/tmp/"+COMMON_NAME+"*.txt"
            try:
                client = paramiko.SSHClient()
                client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

                if AUTH_METHOD == 'Password':
                    client.connect(SERVER_IP, username=USER_NAME, password=PASSWORD,timeout=10)
                    logger.append("trying to connect to {} ({}) with {} and the password...".format(NAME,SERVER_IP,USER_NAME))
                else:
                    private_key = paramiko.RSAKey.from_private_key_file(KEY_FILE_LOCATION)
                    client.connect(SERVER_IP, username=USER_NAME, pkey=private_key,timeout=10)
                    logger.append("trying to connect to {} ({}) using {} and the keyfile...".format(NAME,SERVER_IP,KEY_FILE_LOCATION))

                # ftpclient = client.open_sftp()
                # ftpclient.get(remote_source_file,RunSCP.destination+"/"+COMMON_NAME+"_"+SERVER_IP+".txt")

                with SCPClient(client.get_transport(), sanitize=lambda x: x) as scp:
                    scp.get(remote_path=remote_source_file, local_path=RunSCP.destination)

            except Exception as e:
                logger.append(e)
                logger.append("ERROR! Either failed to connect to {} ({}) or failed to get {} after connecting ".format(NAME,SERVER_IP,remote_source_file))
                ExecuteSCPCommand.success = False
                ExecuteSCPCommand.failed_server.append(NAME)

            finally:
                # ftpclient.close()
                client.close()
                return logger


        with cf.ThreadPoolExecutor() as executor:
            results = [executor.submit(ssh_connect,each_server) for each_server in full_server_detail]
            for f in cf.as_completed(results):
                for logs in range(len(f.result())):
                    dynamic_logger.Logger(root,logtext,str(f.result()[logs])+"\n")

        if(len(ExecuteSCPCommand.failed_server)>0 and len(full_server_detail)==len(ExecuteSCPCommand.failed_server)):
            dynamic_logger.Logger(root,logtext,"The command failed to run on all of the selected servers. Please try again.")
            server_id=[]
            [server_id.append(server[0]) for server in full_server_detail]
            
        elif(len(ExecuteSCPCommand.failed_server)>0):
            dynamic_logger.Logger(root,logtext,"Execution complete, but the command was failed to run on the following server(s):")
            for server in ExecuteSCPCommand.failed_server:
                dynamic_logger.Logger(root,logtext,server)

        else:
            dynamic_logger.Logger(root,logtext,"Execution completed successfully on all the selected servers, deleting entries from pending table.")
            write_transaction_log(full_server_detail)


# ftp_client=ssh_client.open_sftp()
# ftp_client.get(‘remotefileth’,’localfilepath’)
# ftp_client.close()


from tkinter import *
from ui_management import grid, dynamic_logger
from internal_db_management import db_ops
from internal_db_management.get_server_details import GetServerDetails
import paramiko
from datetime import datetime
import tkinter.messagebox as mb
import concurrent.futures as cf
import tkinter.scrolledtext as st

g = grid.Grid()


class RunVmstat:
    def __init__(self,root,server_list,duration,frequency,name):
        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1) 
        Label(root, text="Running VMSTAT...", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)

        full_server_detail = GetServerDetails.get_server_details(server_list,"SERVERS")

        vmstat_command = "vmstat -n "+str(frequency)+" "+str(duration)+" -t>/tmp/"+name+"_"+str(duration)+"s_"+datetime.today().strftime('%Y_%m_%d_%H_%M_%S')+"_$HOSTNAME.txt &"
        ExecuteCommands(root,full_server_detail,vmstat_command,name)


class ExecuteCommands():
    success = True
    failed_server = []

    def __init__(self,root,list_of_servers,command,common_name) -> None:

        def write_transaction_log(time,common_name,command,server_list):
            for server in server_list:
                query = "INSERT INTO VMSTAT (EXECUTION_TIME,COMMON_NAME,COMMAND,SERVER) VALUES(\"{}\",\"{}\",\"{}\",\"{}\");".format(str(time),common_name,command,server)
                db_ops.query_executor(query)

        
        def exit_window():
            answer = mb.askyesno(title='Are you sure?', message="Are you sure that you want to exit? Possible transaction loss if you proceed when a transaction is in progress.")
            if answer:
                root.quit()

        ExecuteCommands.root = root
        logtext = st.ScrolledText(root, width = 75, height = 30, font = ("DejaVu Sans Mono",12))
        logtext.grid(row=g.get_next_row(),column=0,columnspan=10)
        dynamic_logger.Logger(root,logtext,"Running the command...")
        Button(root,text="Exit",command=exit_window).grid(row=g.get_next_row(),column=1)
       
        def ssh_connect(each_server):
            logger = []
            ID, NAME, SERVER_IP, USER_NAME, AUTH_METHOD, KEY_FILE_LOCATION, PASSWORD = each_server
            try:
                client = paramiko.SSHClient()
                client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

                if AUTH_METHOD == 'Password':
                    client.connect(SERVER_IP, username=USER_NAME, password=PASSWORD,timeout=10)
                    logger.append("trying to connect to {} ({}) with {} and the password...".format(NAME,SERVER_IP,USER_NAME))
                else:
                    private_key = paramiko.RSAKey.from_private_key_file(KEY_FILE_LOCATION)
                    client.connect(SERVER_IP, username=USER_NAME, pkey=private_key,timeout=10)
                    logger.append("trying to connect to {} ({}) using {} and the keyfile...".format(NAME,SERVER_IP,KEY_FILE_LOCATION))

                stdin, stdout, stderr = client.exec_command(command)

                
                logger.append("Result of command {}:".format(command))
                logger.append(stdout.read())

            except Exception as e:
                logger.append(e)
                logger.append("ERROR! Either failed to connect to {} ({}) or failed to execute {} after connecting ".format(NAME,SERVER_IP,command))
                ExecuteCommands.success = False
                ExecuteCommands.failed_server.append(NAME)

            finally:
                client.close()
                return logger


        with cf.ThreadPoolExecutor() as executor:
            results = [executor.submit(ssh_connect,each_server) for each_server in list_of_servers]
            for f in cf.as_completed(results):
                for logs in range(len(f.result())):
                    dynamic_logger.Logger(root,logtext,str(f.result()[logs])+"\n")

        if(len(ExecuteCommands.failed_server)>0 and len(list_of_servers)==len(ExecuteCommands.failed_server)):
            dynamic_logger.Logger(root,logtext,"The command failed to run on all of the selected servers. This transaction will not be stored to the database.")
            server_id=[]
            [server_id.append(server[0]) for server in list_of_servers]
            
        elif(len(ExecuteCommands.failed_server)>0):
            dynamic_logger.Logger(root,logtext,"Execution complete, but the command was failed to run on the following server(s):")
            for server in ExecuteCommands.failed_server:
                dynamic_logger.Logger(root,logtext,server)

        else:
            dynamic_logger.Logger(root,logtext,"Execution completed successfully on all the selected servers")
            server_id=[]
            [server_id.append(server[0]) for server in list_of_servers]
            write_transaction_log(datetime.today().strftime('%Y_%m_%d_%H_%M_%S'),common_name,command,server_id)






import tkinter
class Logger():
    def __init__(self,root,text,info) -> None:
        text.configure(state="normal")
        text.insert(tkinter.INSERT,info+"\n")
        text.configure(state="disabled")
        root.update()


from tkinter import *
import ui_management.tk_page_layouts as tk
import ui_management.grid as grid
# from ui_management import tk_page_layouts, grid
import entrypoint.initialize as initialize


g = grid.Grid() 

def main():
    initialize.initialize_db()
    root = Tk()
    root.rowconfigure(0, weight=1)
    root.columnconfigure(0, weight=1)

    frm = Frame(root)
    frm.pack(side=LEFT, padx=20)


    # root.geometry("2000x500")
    root.resizable(0, 0)
    root.title("VMSTAT Automator")

    Label(frm, text="Welcome to VMSTAT automator!", font=('calibre','20','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)

    tk.RecordActions(frm)

    root.mainloop()

if __name__ == "__main__":
    main()


import sqlite3,os
 
def connect_to_db():
    db_location = os.path.join(os.getcwd(),"bigtest_automator", "serverdb.db")

    try:
        con = sqlite3.connect(db_location,isolation_level=None)
        if con is not None:
            return con.cursor(), con
        else:
            raise ValueError("Could not establish a DB connection by using the DB file "+db_location)
    except sqlite3.Error as e:
        print(e)

def query_executor(query):
    #Get connection cursor
    cursor,con = connect_to_db()

    if cursor is not None:
        try:
            result_set = cursor.execute(query)
            con.commit()
            return result_set

        except sqlite3.Error as e:
            print(e)
            raise ValueError("Error in executing the Query")
     
    else:
        raise ValueError("Could not get a connection object from the DB")

def main_table_creator():
    main_table_query = """
    CREATE TABLE IF NOT EXISTS SERVERS (
        ID integer PRIMARY KEY AUTOINCREMENT,
        NAME text NOT NULL,
        SERVER_IP text NOT NULL,
        USER_NAME text NOT NULL,
        AUTH_METHOD text NOT NULL,
        KEY_FILE_LOCATION text,
        PASSWORD text
    );"""

    query_executor(main_table_query)



from setuptools import setup, find_packages

setup(
    name='bigtest_automator',
    version='0.8',
    author='Shantharam Puranik M',
    author_email='shantharam.puranik@ellucian.com',
    description='A small tool that helps to run and collect performance related metrics such as VMSTAT and AWR. Highly specific to my ORG, might not be useful for general public. SORRY! - Noob.',
    packages=find_packages(),
    install_requires=[
        'scp',
        'paramiko',
        'oracledb'
    ],
    entry_points={
        'console_scripts': [
            'reportscraper=entrypoint.main_class:main',
            # Add other command-line tools here
        ],
    },
)



from tkinter import *
from tkinter import messagebox
from datetime import datetime,timedelta
import time,math
import tkinter.messagebox as mb
import tkinter.scrolledtext as st
from ui_management import dynamic_logger,grid
from internal_db_management import db_ops, get_server_details
from oracle_utils import oracle_awr

g = grid.Grid()


class TakeSnapshot():
    success = True
    failed_server = []
    logger = []
    snap_id = ""
    snap_time = ""

    def __init__(self,root,server_list,when_to_take,how_much_to_wait):

        for item in root.winfo_children():
            item.destroy()
        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1)

        Label(root, text="Take DB snapshot", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root, text="DB Connection Details", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        Label(root, text="Enter the following details to connect to the DB", font=('calibre','13','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        message = """
            Note - Note - Multiple selections are not supported in this feature!
            """

        Label(root, text=message, font=('calibre','13')).grid(row=g.get_next_row(),column=0,columnspan=10)

        g.get_blank_lines(root,1)


        db_user_grid_row = g.get_next_row()
        db_pass_grid_row = g.get_next_row()
        db_port_grid_row = g.get_next_row()
        db_sid_grid_row = g.get_next_row()


        Label(root,text="Database username: ",pady=10).grid(row=db_user_grid_row,column=0,sticky="w")
        Label(root,text="Database password: ",pady=10).grid(row=db_pass_grid_row,column=0,sticky="w")
        Label(root,text="Port: ",pady=10).grid(row=db_port_grid_row,column=0,sticky="w")
        Label(root,text="SID: ",pady=10).grid(row=db_sid_grid_row,column=0,sticky="w")


        username_entry = Entry(root,width=40,borderwidth=2)
        username_entry.grid(row=db_user_grid_row,column=1,padx=10,pady=10)
        password_entry = Entry(root,show='*',width=40,borderwidth=2)
        password_entry.grid(row=db_pass_grid_row,column=1,padx=10,pady=10)
        port_entry = Entry(root,width=40,borderwidth=2)
        port_entry.grid(row=db_port_grid_row,column=1,padx=10,pady=10)
        sid_entry = Entry(root,width=40,borderwidth=2)
        sid_entry.grid(row=db_sid_grid_row,column=1,padx=10,pady=10)

        def submit_info():
            if not port_entry.get().isdigit():
                messagebox.showerror("Error","Please check your input. Port must be a number")
            elif(' ' in username_entry.get() or ' ' in password_entry.get() or ' ' in sid_entry.get()):
                messagebox.showerror("Error","Please enter valid values")
            elif(len(username_entry.get()) < 1 or len(password_entry.get()) < 1  or len(sid_entry.get()) < 1 ):
                messagebox.showerror("Error","Please enter valid values")
            else:
                full_server_detail = get_server_details.GetServerDetails.get_server_details(server_list,"SERVERS")
                TakeSnapshot.get_db_snapshot(root,full_server_detail,username_entry.get(), password_entry.get(), port_entry.get(), sid_entry.get(),when_to_take,how_much_to_wait)
                
        button_grid_row = g.get_next_row()
        Button(root,text="Submit",command=submit_info).grid(row=button_grid_row,column=0)
        Button(root,text="Exit",command=root.quit).grid(row=button_grid_row,column=1)
    

    def get_db_snapshot(root,server_list,db_username,db_password,db_port,db_sid,when_to_take,how_much_to_wait):

        server_ip = []
        for each_server in server_list:
            server_ip.append(each_server[2])

        def write_transaction_log(server_id_list):
            for server in server_id_list:
                query = "INSERT INTO AWR (SERVER_ID, SNAP_ID, SNAP_TIME) VALUES (\"{}\",\"{}\",\"{}\")".format(server,TakeSnapshot.snap_id,TakeSnapshot.snap_time)
                try:
                    db_ops.query_executor(query)
                except Exception as e:
                    print("Failed to write to the DB")
                    print(e)

        def exit_window():
            answer = mb.askyesno(title='Are you sure?', message="Are you sure that you want to exit? Possible transaction loss if you proceed when a transaction is in progress.")
            if answer:
                root.quit()

        for item in root.winfo_children():
            item.destroy()

        root.rowconfigure(0, weight=1)
        root.columnconfigure(0, weight=1) 

        Label(root, text="Taking a snapshot...", font=('calibre','15','bold')).grid(row=g.get_next_row(),column=0,columnspan=10)
        logtext = st.ScrolledText(root, width = 75, height = 30, font = ("DejaVu Sans Mono",12))
        logtext.grid(row=g.get_next_row(),column=0,columnspan=10)
        dynamic_logger.Logger(root,logtext,"Running the command...")
        Button(root,text="Exit",command=exit_window).grid(row=g.get_next_row(),column=1)

        def run_snapshot(each_server,when_to_take,how_much_to_wait):

            if when_to_take == "now":
                TakeSnapshot.snap_id, TakeSnapshot.snap_time, TakeSnapshot.logger = oracle_awr.take_db_snapshot(root,each_server,db_username,db_password,db_port,db_sid,when_to_take,how_much_to_wait)

                if TakeSnapshot.snap_id == "Error" or TakeSnapshot.snap_time == "Error":
                    TakeSnapshot.success = False
                    TakeSnapshot.failed_server.append(each_server)
                return TakeSnapshot.logger
            
            else:

                current_time = datetime.now()
                target_time = current_time + timedelta(minutes=int(how_much_to_wait))

                while(current_time < target_time):
                    time.sleep(30)
                    current_time = datetime.now()
                    dynamic_logger.Logger(root,logtext,"Waiting for {} more hours...".format(target_time-current_time))

                    # TakeSnapshot.logger.append("Waiting for {} more minutes...".format(target_time-current_time))
                TakeSnapshot.logger.append(run_snapshot(each_server,"now",0))
                return TakeSnapshot.logger
            
        for each_server in server_ip:
            logger = run_snapshot(each_server,when_to_take,how_much_to_wait)
            for each_line in logger:
                dynamic_logger.Logger(root,logtext,each_line)

        # with cf.ThreadPoolExecutor() as executor:
        #     results = [executor.submit(run_snapshot,each_server,when_to_take,how_much_to_wait) for each_server in server_ip]
        #     for f in cf.as_completed(results):
        #         for logs in range(len(f.result())):
        #             dynamic_logger.Logger(root,logtext,str(f.result()[logs])+"\n")

        if(len(TakeSnapshot.failed_server)>0 and len(server_ip)==len(TakeSnapshot.failed_server)):
            dynamic_logger.Logger(root,logtext,"Failed to generate snapshot on all of the seclected servers. This transaction will not be stored to the database.")
            server_id=[]
            [server_id.append(server[0]) for server in server_list]
            
        elif(len(TakeSnapshot.failed_server)>0):
            dynamic_logger.Logger(root,logtext,"Execution complete, but the snapshot was failed to create on the following server(s):")
            for server in TakeSnapshot.failed_server:
                dynamic_logger.Logger(root,logtext,server)

        else:
            dynamic_logger.Logger(root,logtext,"Snapshot was created successfully on all the selected servers")
            server_id=[]
            [server_id.append(server[0]) for server in server_list]
            write_transaction_log(server_id)
       


import remote_oracle_db_connector, oracledb,ast

host = "34.194.224.7"
username = "baninst1"
password = "u_pick_it"
sid = "DPD8"
port = 8000
begin_snap = "36879"
end_snap = "36880"
filepath = "/tmp"

try:

    params = oracledb.ConnectParams(host=host, port=port, service_name=sid)
    connection = oracledb.connect(user=username, password=password, params=params)
    cursor = connection.cursor()
    cursor.arraysize = 10000
    # rs = remote_oracle_db_connector.execute_query("select snap_id, snap_level, to_char(begin_interval_time, 'dd/mm/yy hh24:mi:ss') from dba_hist_snapshot order by 1 desc fetch first 1 rows only",host,username,password,port,sid)
    # query = "dbms_output.put_line(exec dbms_workload_repository.create_snapshot);"

    # query = """
    #     begin
    #         dbms_output.put_line(dbms_workload_repository.create_snapshot);
    #     end;
    # """

    # rs = remote_oracle_db_connector.execute_query("sql","SELECT DBID FROM V$DATABASE",host,username,password,port,sid)
    rs = cursor.execute("SELECT DBID FROM V$DATABASE")
    result_set = []
    for row in rs:
        result_set.append(row)
    db_id = result_set[0][0]
    query = """
            SELECT output FROM TABLE
            (dbms_workload_repository.awr_report_html
                ({},{},{},{})
            )
            """.format(db_id,str(1),begin_snap,end_snap)
    # rs = remote_oracle_db_connector.execute_query("plsql",query,host,username,password,port,sid)
    cursor.execute(query)
    result = cursor.fetchall()

    mainarray = []

    for i in range(len(result)):
        mainarray.append(result[i][0])

    # print(mainarray)

    print(len(mainarray))

    with open("/Users/shantharamp/Downloads/adf.html","a") as f:
        for j in range(len(mainarray)):
            f.write(str(mainarray[j]))

    


    # print(rs)
except Exception as e:
    print(e)
    # raise ValueError("error while exeucing the query")



